package application;

import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.text.Font;

import java.io.PrintStream;
import java.net.URL;
import java.util.ResourceBundle;

import RalliJava.Henkilo;
import RalliJava.Rekisteri;
import RalliJava.SailoException;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.TextAreaOutputStream;

/**
 * @author idamk
 * @version 5.2.2025
 *
 */
public class RalliController implements Initializable {
    
    @FXML private TextField textrajaus; //hakutulosten rajaus'
    @FXML private ScrollPane panelHenkilo;
    @FXML private ListChooser<Henkilo> chooserHenkilot;
	
    /*
     * Alustaa vasemman reunan listan tyhjäksi.
     * TODO: lisätyn tiedot tulisi näkyviin keskelle tekstikenttään
     */
    @Override
   public void initialize(URL url, ResourceBundle bundle) {
        alusta();
    }
    
    /*
     * Lisää uuden, avaa uuden ikkunan, missä pääsee lisäämään joko uuden rallin tai uuden henkilön
     */
    @FXML private void handleUusi() {
        //Dialogs.showQuestionDialog("Poisto?", "Poistetaanko?", "Kyllä", "Ei"); //Otsikko, teksti, nappi1, nappi2
        //ModalController.showModal(RalliController.class.getResource("uudenlisays.fxml"), "Lisää uusi", null, ""); //Modal controller -> ei voida tehdä muuta
        //TODO: miten toteutus uuden rallin ja uuden henkilön lisäämiselle, pitää varmaan olla eri ikkunat
        uusiHenkilo();
    }
    
    
    private void uusiHenkilo() {
        Henkilo uusi = new Henkilo();
        uusi.rekisteroi(); //TODO rekisteröinti vasta kun käyttäjä tallentaa uuden tiedot
        uusi.vastaahlo(); //TODO että käyttäjä saa täyttää itse
        //uusi.tulosta(System.out);
        try {
            rekisteri.lisaa(uusi);
        } catch (SailoException e) { //catch koska määrtelty että rekisteri saattaa heittää poikkeuksen
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
        }
        hae(uusi.getTunnusNro()); 
    }
    
    /**
     * @return true
     */
    public boolean voikoSulkea() {
        //tallenna();
        return true;
    }
    
    
/*================================================================*/
 
    //private Henkilo henkiloKohdalla;
     private Rekisteri rekisteri;
     private TextArea areaHenkilo = new TextArea(); //TODO poista lopuksi!!
     
     
     /*
      * Alustaa vasemman teksti-ikkunan tyhjäksi
      * asettaa myös kuuntelijan listalle
      */
     private void alusta() {
         panelHenkilo.setContent(areaHenkilo);
         areaHenkilo.setFont(new Font("Courier New", 12));
         panelHenkilo.setFitToHeight(true);
         chooserHenkilot.clear();
         chooserHenkilot.addSelectionListener(e -> naytaHenkilo()); // e on lambdalauseke
         //TODO: saa uudet henkilöt näkymään listassa
     }
     
    
    /**
     * @param rekisteri mitä käytetään
     */
     public void setRekisteri(Rekisteri rekisteri){
        this.rekisteri = rekisteri;
        //naytaHenkilo();
     }
    
    
     /*
     * Haetaan henkilöt uudelleen
     * @param idnro henkilön tunniste
     */
     private void hae(int idnro) {
        chooserHenkilot.clear();
        
        int index = 0;
        for(int i = 0; i < rekisteri.getHenkiloita(); i++) {    //käydään läpi henkilöt
            Henkilo henkilo = rekisteri.annaHenkilo(i);         //indeksin perusteella käy läpi
            if (henkilo.getTunnusNro() == idnro) index = i;     //samalla kun käy läpi henkilöitä, etsii oikeaa indeksiä
            chooserHenkilot.add(henkilo.getNimi(), henkilo);    // viimeinen kohta linkittää juuri tämän henkilön chooseriin
        }
        chooserHenkilot.setSelectedIndex(index); 
     }

    
    /*
     * Avaa ikkunan, missä pääsee muokkaamaan valitun henkilön tai rallin tietoja
     */
    @FXML private void handleMuokkaa() {
        //Dialogs.showMessageDialog("Muokkaa valittua henkilöä/rallia, ei osaa vielä :((");
        ModalController.showModal(RalliController.class.getResource("muokkausRalli.fxml"), "Muokkaa", null, "");
    }
    
    
    /*
     * Poistuu ikkunasta
     */
    @FXML private void handlePoistu() {
        //TODO:tallenna();
        Platform.exit();
    }

    
    /*
     * Näyttää valitun jäsenen tiedot väliaikaisesti
     * @param rekisteri 
     */ 
    private void naytaHenkilo() {
        Henkilo henkiloKohdalla = chooserHenkilot.getSelectedObject();
        
        if(henkiloKohdalla == null) return; //jos ei oo valittuna ketään, poistutaan
        
        areaHenkilo.setText("");  //tyhjennetään entinen näkyviltä
        try (PrintStream out = TextAreaOutputStream.getTextPrintStream(areaHenkilo)) {
            henkiloKohdalla.tulosta(out);
        }
    }
   
    
    /*
     * 
     */
    @FXML private void handleRajaus() {
        Dialogs.showMessageDialog("Rajaa hakutuloksia, ei osaa vielä :((");
    }
}
